@@ -26,7 +26,9 @@
 #define GF_CRTC_PRIVATE_H
 
 #include <glib-object.h>
+#include <stdint.h>
 
+#include "gf-crtc-mode-private.h"
 #include "gf-gpu-private.h"
 #include "gf-monitor-manager-enums-private.h"
 #include "gf-monitor-manager-types-private.h"
@@ -42,41 +44,6 @@ typedef struct
   GfCrtcMode         *mode;
 } GfCrtcConfig;
 
-struct _GfCrtc
-{
-  GObject         parent;
-
-  GfGpu          *gpu;
-
-  glong           crtc_id;
-  guint           all_transforms;
-
-  GfCrtcConfig   *config;
-
-  /* Used when changing configuration */
-  gboolean        is_dirty;
-
-  gpointer        driver_private;
-  GDestroyNotify  driver_notify;
-};
-
-struct _GfCrtcMode
-{
-  GObject         parent;
-
-  /* The low-level ID of this mode, used to apply back configuration */
-  glong           mode_id;
-  gchar          *name;
-
-  gint            width;
-  gint            height;
-  gfloat          refresh_rate;
-  GfCrtcModeFlag  flags;
-
-  gpointer        driver_private;
-  GDestroyNotify  driver_notify;
-};
-
 typedef struct
 {
   GfCrtc             *crtc;
@@ -84,22 +51,31 @@ typedef struct
   GfRectangle         layout;
   GfMonitorTransform  transform;
   GPtrArray          *outputs;
-} GfCrtcInfo;
+} GfCrtcAssignment;
 
 #define GF_TYPE_CRTC (gf_crtc_get_type ())
-G_DECLARE_FINAL_TYPE (GfCrtc, gf_crtc, GF, CRTC, GObject)
+G_DECLARE_DERIVABLE_TYPE (GfCrtc, gf_crtc, GF, CRTC, GObject)
 
-#define GF_TYPE_CRTC_MODE (gf_crtc_mode_get_type ())
-G_DECLARE_FINAL_TYPE (GfCrtcMode, gf_crtc_mode, GF, CRTC_MODE, GObject)
+struct _GfCrtcClass
+{
+  GObjectClass parent_class;
+};
 
-GfGpu *gf_crtc_get_gpu      (GfCrtc             *crtc);
+uint64_t            gf_crtc_get_id             (GfCrtc             *self);
 
-void   gf_crtc_set_config   (GfCrtc             *crtc,
-                             GfRectangle        *layout,
-                             GfCrtcMode         *mode,
-                             GfMonitorTransform  transform);
+GfGpu              *gf_crtc_get_gpu            (GfCrtc             *self);
 
-void   gf_crtc_unset_config (GfCrtc             *crtc);
+GfMonitorTransform  gf_crtc_get_all_transforms (GfCrtc             *self);
+
+void                gf_crtc_set_config         (GfCrtc             *self,
+                                                GfRectangle        *layout,
+                                                GfCrtcMode         *mode,
+                                                GfMonitorTransform  transform);
+
+void                gf_crtc_unset_config       (GfCrtc             *self);
+
+
+const GfCrtcConfig *gf_crtc_get_config         (GfCrtc             *self);
 
 G_END_DECLS
 
