@@ -27,24 +27,23 @@
 
 #include <X11/Xlib-xcb.h>
 
+#include "gf-backend-private.h"
+#include "gf-monitor-manager-xrandr-private.h"
 #include "gf-output-private.h"
 
 #define ALL_ROTATIONS (RR_Rotate_0 | RR_Rotate_90 | RR_Rotate_180 | RR_Rotate_270)
-#define ALL_TRANSFORMS ((1 << (GF_MONITOR_TRANSFORM_FLIPPED_270 + 1)) - 1)
 
-typedef struct
+struct _GfCrtcXrandr
 {
+  GfCrtc              parent;
+
   GfRectangle         rect;
   GfMonitorTransform  transform;
 
   GfCrtcMode         *current_mode;
-} GfCrtcXrandr;
+};
 
-static void
-gf_crtc_destroy_notify (GfCrtc *crtc)
-{
-  g_free (crtc->driver_private);
-}
+G_DEFINE_TYPE (GfCrtcXrandr, gf_crtc_xrandr, GF_TYPE_CRTC)
 
 static GfMonitorTransform
 gf_monitor_transform_from_xrandr (Rotation rotation)
@@ -97,7 +96,7 @@ gf_monitor_transform_from_xrandr_all (Rotation rotation)
   /* All rotations and one reflection -> all of them by composition */
   if ((rotation & ALL_ROTATIONS) &&
       ((rotation & RR_Reflect_X) || (rotation & RR_Reflect_Y)))
-    return ALL_TRANSFORMS;
+    return GF_MONITOR_ALL_TRANSFORMS;
 
   ret = 1 << GF_MONITOR_TRANSFORM_NORMAL;
   if (rotation & RR_Rotate_90)
@@ -118,21 +117,39 @@ gf_monitor_transform_from_xrandr_all (Rotation rotation)
   return ret;
 }
 
-GfCrtc *
-gf_create_xrandr_crtc (GfGpuXrandr        *gpu_xrandr,
-                       XRRCrtcInfo        *xrandr_crtc,
-                       RRCrtc              crtc_id,
-                       XRRScreenResources *resources)
+static void
+gf_crtc_xrandr_class_init (GfCrtcXrandrClass *self_class)
+{
+}
+
+static void
+gf_crtc_xrandr_init (GfCrtcXrandr *self)
+{
+}
+
+GfCrtcXrandr *
+gf_crtc_xrandr_new (GfGpuXrandr        *gpu_xrandr,
+                    XRRCrtcInfo        *xrandr_crtc,
+                    RRCrtc              crtc_id,
+                    XRRScreenResources *resources)
 
 {
-  GfCrtc *crtc;
+  GfGpu *gpu;
+  GfMonitorTransform all_transforms;
   GfCrtcXrandr *crtc_xrandr;
   unsigned int i;
   GList *modes;
 
-  crtc = g_object_new (GF_TYPE_CRTC, NULL);
+  gpu = GF_GPU (gpu_xrandr);
+
+  all_transforms = gf_monitor_transform_from_xrandr_all (xrandr_crtc->rotations);
+
+  crtc_xrandr = g_object_new (GF_TYPE_CRTC_XRANDR,
+                              "id", crtc_id,
+                              "gpu", gpu,
+                              "all-transforms", all_transforms,
+                              NULL);
 
-  crtc_xrandr = g_new0 (GfCrtcXrandr, 1);
   crtc_xrandr->rect = (GfRectangle) {
     .x = xrandr_crtc->x,
     .y = xrandr_crtc->y,
@@ -141,15 +158,7 @@ gf_create_xrandr_crtc (GfGpuXrandr        *gpu_xrandr,
   };
   crtc_xrandr->transform = gf_monitor_transform_from_xrandr (xrandr_crtc->rotation);
 
-  crtc->driver_private = crtc_xrandr;
-  crtc->driver_notify = (GDestroyNotify) gf_crtc_destroy_notify;
-
-  crtc->gpu = GF_GPU (gpu_xrandr);
-  crtc->crtc_id = crtc_id;
-  crtc->is_dirty = FALSE;
-  crtc->all_transforms = gf_monitor_transform_from_xrandr_all (xrandr_crtc->rotations);
-
-  modes = gf_gpu_get_modes (crtc->gpu);
+  modes = gf_gpu_get_modes (gpu);
   for (i = 0; i < (guint) resources->nmode; i++)
     {
       if (resources->modes[i].id == xrandr_crtc->mode)
@@ -161,17 +170,17 @@ gf_create_xrandr_crtc (GfGpuXrandr        *gpu_xrandr,
 
   if (crtc_xrandr->current_mode)
     {
-      gf_crtc_set_config (crtc,
+      gf_crtc_set_config (GF_CRTC (crtc_xrandr),
                           &crtc_xrandr->rect,
                           crtc_xrandr->current_mode,
                           crtc_xrandr->transform);
     }
 
-  return crtc;
+  return crtc_xrandr;
 }
 
 gboolean
-gf_crtc_xrandr_set_config (GfCrtc               *crtc,
+gf_crtc_xrandr_set_config (GfCrtcXrandr         *self,
                            xcb_randr_crtc_t      xrandr_crtc,
                            xcb_timestamp_t       timestamp,
                            int                   x,
@@ -184,6 +193,7 @@ gf_crtc_xrandr_set_config (GfCrtc               *crtc,
 {
   GfGpu *gpu;
   GfGpuXrandr *gpu_xrandr;
+  GfBackend *backend;
   GfMonitorManager *monitor_manager;
   GfMonitorManagerXrandr *monitor_manager_xrandr;
   Display *xdisplay;
@@ -194,10 +204,12 @@ gf_crtc_xrandr_set_config (GfCrtc               *crtc,
   xcb_randr_set_crtc_config_reply_t *reply;
   xcb_generic_error_t *xcb_error;
 
-  gpu = gf_crtc_get_gpu (crtc);
+  gpu = gf_crtc_get_gpu (GF_CRTC (self));
   gpu_xrandr = GF_GPU_XRANDR (gpu);
 
-  monitor_manager = gf_gpu_get_monitor_manager (gpu);
+  backend = gf_gpu_get_backend (gpu);
+
+  monitor_manager = gf_backend_get_monitor_manager (backend);
   monitor_manager_xrandr = GF_MONITOR_MANAGER_XRANDR (monitor_manager);
 
   xdisplay = gf_monitor_manager_xrandr_get_xdisplay (monitor_manager_xrandr);
@@ -227,35 +239,32 @@ gf_crtc_xrandr_set_config (GfCrtc               *crtc,
 }
 
 gboolean
-gf_crtc_xrandr_is_assignment_changed (GfCrtc     *crtc,
-                                      GfCrtcInfo *crtc_info)
+gf_crtc_xrandr_is_assignment_changed (GfCrtcXrandr     *self,
+                                      GfCrtcAssignment *crtc_assignment)
 {
-  GfCrtcXrandr *crtc_xrandr;
   unsigned int i;
 
-  crtc_xrandr = crtc->driver_private;
-
-  if (crtc_xrandr->current_mode != crtc_info->mode)
+  if (self->current_mode != crtc_assignment->mode)
     return TRUE;
 
-  if (crtc_xrandr->rect.x != crtc_info->layout.x)
+  if (self->rect.x != crtc_assignment->layout.x)
     return TRUE;
 
-  if (crtc_xrandr->rect.y != crtc_info->layout.y)
+  if (self->rect.y != crtc_assignment->layout.y)
     return TRUE;
 
-  if (crtc_xrandr->transform != crtc_info->transform)
+  if (self->transform != crtc_assignment->transform)
     return TRUE;
 
-  for (i = 0; i < crtc_info->outputs->len; i++)
+  for (i = 0; i < crtc_assignment->outputs->len; i++)
     {
       GfOutput *output;
       GfCrtc *assigned_crtc;
 
-      output = ((GfOutput **) crtc_info->outputs->pdata)[i];
+      output = ((GfOutput **) crtc_assignment->outputs->pdata)[i];
       assigned_crtc = gf_output_get_assigned_crtc (output);
 
-      if (assigned_crtc != crtc)
+      if (assigned_crtc != GF_CRTC (self))
         return TRUE;
     }
 
@@ -263,11 +272,7 @@ gf_crtc_xrandr_is_assignment_changed (GfCrtc     *crtc,
 }
 
 GfCrtcMode *
-gf_crtc_xrandr_get_current_mode (GfCrtc *crtc)
+gf_crtc_xrandr_get_current_mode (GfCrtcXrandr *self)
 {
-  GfCrtcXrandr *crtc_xrandr;
-
-  crtc_xrandr = crtc->driver_private;
-
-  return crtc_xrandr->current_mode;
+  return self->current_mode;
 }
