@@ -63,13 +63,6 @@ struct _GfMonitorManagerXrandr
   gboolean          has_randr15;
   GHashTable       *tiled_monitor_atoms;
 
-  /*
-   * The X server deals with multiple GPUs for us, soe just see what the X
-   * server gives us as one single GPU, even though it may actually be backed
-   * by multiple.
-   */
-  GfGpu            *gpu;
-
   Time              last_xrandr_set_timestamp;
 
   gfloat           *supported_scales;
@@ -189,7 +182,8 @@ xrandr_set_crtc_config (GfMonitorManagerXrandr *xrandr,
 {
   xcb_timestamp_t new_timestamp;
 
-  if (!gf_crtc_xrandr_set_config (crtc, xrandr_crtc, timestamp,
+  if (!gf_crtc_xrandr_set_config (GF_CRTC_XRANDR (crtc),
+                                  xrandr_crtc, timestamp,
                                   x, y, mode, rotation,
                                   outputs, n_outputs,
                                   &new_timestamp))
@@ -202,31 +196,34 @@ xrandr_set_crtc_config (GfMonitorManagerXrandr *xrandr,
 }
 
 static gboolean
-is_crtc_assignment_changed (GfCrtc      *crtc,
-                            GfCrtcInfo **crtc_infos,
-                            guint        n_crtc_infos)
+is_crtc_assignment_changed (GfCrtc            *crtc,
+                            GfCrtcAssignment **crtc_assignments,
+                            unsigned int       n_crtc_assignments)
 {
   guint i;
 
-  for (i = 0; i < n_crtc_infos; i++)
+  for (i = 0; i < n_crtc_assignments; i++)
     {
-      GfCrtcInfo *crtc_info = crtc_infos[i];
+      GfCrtcAssignment *crtc_assignment;
 
-      if (crtc_info->crtc != crtc)
+      crtc_assignment = crtc_assignments[i];
+
+      if (crtc_assignment->crtc != crtc)
         continue;
 
-      return gf_crtc_xrandr_is_assignment_changed (crtc, crtc_info);
+      return gf_crtc_xrandr_is_assignment_changed (GF_CRTC_XRANDR (crtc),
+                                                   crtc_assignment);
     }
 
-  return !!gf_crtc_xrandr_get_current_mode (crtc);
+  return !!gf_crtc_xrandr_get_current_mode (GF_CRTC_XRANDR (crtc));
 }
 
 static gboolean
-is_output_assignment_changed (GfOutput      *output,
-                              GfCrtcInfo   **crtc_infos,
-                              guint          n_crtc_infos,
-                              GfOutputInfo **output_infos,
-                              guint          n_output_infos)
+is_output_assignment_changed (GfOutput            *output,
+                              GfCrtcAssignment   **crtc_assignments,
+                              guint                n_crtc_assignments,
+                              GfOutputAssignment **output_assignments,
+                              guint                n_output_assignments)
 {
   gboolean output_is_found;
   GfCrtc *assigned_crtc;
@@ -234,20 +231,22 @@ is_output_assignment_changed (GfOutput      *output,
 
   output_is_found = FALSE;
 
-  for (i = 0; i < n_output_infos; i++)
+  for (i = 0; i < n_output_assignments; i++)
     {
-      GfOutputInfo *output_info = output_infos[i];
+      GfOutputAssignment *output_assignment;
 
-      if (output_info->output != output)
+      output_assignment = output_assignments[i];
+
+      if (output_assignment->output != output)
         continue;
 
-      if (output->is_primary != output_info->is_primary)
+      if (gf_output_is_primary (output) != output_assignment->is_primary)
         return TRUE;
 
-      if (output->is_presentation != output_info->is_presentation)
+      if (gf_output_is_presentation (output) != output_assignment->is_presentation)
         return TRUE;
 
-      if (output->is_underscanning != output_info->is_underscanning)
+      if (gf_output_is_underscanning (output) != output_assignment->is_underscanning)
         return TRUE;
 
       output_is_found = TRUE;
@@ -258,19 +257,21 @@ is_output_assignment_changed (GfOutput      *output,
   if (!output_is_found)
     return assigned_crtc != NULL;
 
-  for (i = 0; i < n_crtc_infos; i++)
+  for (i = 0; i < n_crtc_assignments; i++)
     {
-      GfCrtcInfo *crtc_info = crtc_infos[i];
+      GfCrtcAssignment *crtc_assignment;
       guint j;
 
-      for (j = 0; j < crtc_info->outputs->len; j++)
+      crtc_assignment = crtc_assignments[i];
+
+      for (j = 0; j < crtc_assignment->outputs->len; j++)
         {
-          GfOutput *crtc_info_output;
+          GfOutput *crtc_assignment_output;
 
-          crtc_info_output = ((GfOutput**) crtc_info->outputs->pdata)[j];
+          crtc_assignment_output = ((GfOutput**) crtc_assignment->outputs->pdata)[j];
 
-          if (crtc_info_output == output &&
-              crtc_info->crtc == assigned_crtc)
+          if (crtc_assignment_output == output &&
+              crtc_assignment->crtc == assigned_crtc)
             return FALSE;
         }
     }
@@ -278,35 +279,49 @@ is_output_assignment_changed (GfOutput      *output,
   return TRUE;
 }
 
+static GfGpu *
+get_gpu (GfMonitorManagerXrandr *self)
+{
+  GfMonitorManager *manager;
+  GfBackend *backend;
+
+  manager = GF_MONITOR_MANAGER (self);
+  backend = gf_monitor_manager_get_backend (manager);
+
+  return GF_GPU (gf_backend_get_gpus (backend)->data);
+}
+
 static gboolean
-is_assignments_changed (GfMonitorManager  *manager,
-                        GfCrtcInfo       **crtc_infos,
-                        guint              n_crtc_infos,
-                        GfOutputInfo     **output_infos,
-                        guint              n_output_infos)
+is_assignments_changed (GfMonitorManager    *manager,
+                        GfCrtcAssignment   **crtc_assignments,
+                        guint                n_crtc_assignments,
+                        GfOutputAssignment **output_assignments,
+                        guint                n_output_assignments)
 {
   GfMonitorManagerXrandr *manager_xrandr;
+  GfGpu *gpu;
   GList *l;
 
   manager_xrandr = GF_MONITOR_MANAGER_XRANDR (manager);
+  gpu = get_gpu (manager_xrandr);
 
-  for (l = gf_gpu_get_crtcs (manager_xrandr->gpu); l; l = l->next)
+  for (l = gf_gpu_get_crtcs (gpu); l; l = l->next)
     {
       GfCrtc *crtc = l->data;
 
-      if (is_crtc_assignment_changed (crtc, crtc_infos, n_crtc_infos))
+      if (is_crtc_assignment_changed (crtc, crtc_assignments, n_crtc_assignments))
         return TRUE;
     }
 
-  for (l = gf_gpu_get_outputs (manager_xrandr->gpu); l; l = l->next)
+  for (l = gf_gpu_get_outputs (gpu); l; l = l->next)
     {
       GfOutput *output = l->data;
 
       if (is_output_assignment_changed (output,
-                                        crtc_infos,
-                                        n_crtc_infos,
-                                        output_infos,
-                                        n_output_infos))
+                                        crtc_assignments,
+                                        n_crtc_assignments,
+                                        output_assignments,
+                                        n_output_assignments))
         return TRUE;
     }
 
@@ -363,19 +378,26 @@ gf_monitor_transform_to_xrandr (GfMonitorTransform transform)
 }
 
 static void
-apply_crtc_assignments (GfMonitorManager  *manager,
-                        gboolean           save_timestamp,
-                        GfCrtcInfo       **crtcs,
-                        guint              n_crtcs,
-                        GfOutputInfo     **outputs,
-                        guint              n_outputs)
+apply_crtc_assignments (GfMonitorManager    *manager,
+                        gboolean             save_timestamp,
+                        GfCrtcAssignment   **crtcs,
+                        guint                n_crtcs,
+                        GfOutputAssignment **outputs,
+                        guint                n_outputs)
 {
   GfMonitorManagerXrandr *xrandr;
+  GfGpu *gpu;
+  GList *to_configure_outputs;
+  GList *to_disable_crtcs;
   gint width, height, width_mm, height_mm;
   guint i;
   GList *l;
 
   xrandr = GF_MONITOR_MANAGER_XRANDR (manager);
+  gpu = get_gpu (xrandr);
+
+  to_configure_outputs = g_list_copy (gf_gpu_get_outputs (gpu));
+  to_disable_crtcs = g_list_copy (gf_gpu_get_crtcs (gpu));
 
   XGrabServer (xrandr->xdisplay);
 
@@ -383,16 +405,16 @@ apply_crtc_assignments (GfMonitorManager  *manager,
   width = 0; height = 0;
   for (i = 0; i < n_crtcs; i++)
     {
-      GfCrtcInfo *crtc_info = crtcs[i];
-      GfCrtc *crtc = crtc_info->crtc;
+      GfCrtcAssignment *crtc_assignment = crtcs[i];
+      GfCrtc *crtc = crtc_assignment->crtc;
 
-      crtc->is_dirty = TRUE;
-
-      if (crtc_info->mode == NULL)
+      if (crtc_assignment->mode == NULL)
         continue;
 
-      width = MAX (width, crtc_info->layout.x + crtc_info->layout.width);
-      height = MAX (height, crtc_info->layout.y + crtc_info->layout.height);
+      to_disable_crtcs = g_list_remove (to_disable_crtcs, crtc);
+
+      width = MAX (width, crtc_assignment->layout.x + crtc_assignment->layout.width);
+      height = MAX (height, crtc_assignment->layout.y + crtc_assignment->layout.height);
     }
 
   /* Second disable all newly disabled CRTCs, or CRTCs that in the previous
@@ -402,24 +424,24 @@ apply_crtc_assignments (GfMonitorManager  *manager,
    */
   for (i = 0; i < n_crtcs; i++)
     {
-      GfCrtcInfo *crtc_info = crtcs[i];
-      GfCrtc *crtc = crtc_info->crtc;
-      GfCrtcConfig *crtc_config;
+      GfCrtcAssignment *crtc_assignment = crtcs[i];
+      GfCrtc *crtc = crtc_assignment->crtc;
+      const GfCrtcConfig *crtc_config;
       int x2, y2;
 
-      crtc_config = crtc->config;
+      crtc_config = gf_crtc_get_config (crtc);
       if (crtc_config == NULL)
         continue;
 
       x2 = crtc_config->layout.x + crtc_config->layout.width;
       y2 = crtc_config->layout.y + crtc_config->layout.height;
 
-      if (crtc_info->mode == NULL || x2 > width || y2 > height)
+      if (crtc_assignment->mode == NULL || x2 > width || y2 > height)
         {
           xrandr_set_crtc_config (xrandr,
                                   crtc,
                                   save_timestamp,
-                                  (xcb_randr_crtc_t) crtc->crtc_id,
+                                  (xcb_randr_crtc_t) gf_crtc_get_id (crtc),
                                   XCB_CURRENT_TIME,
                                   0, 0, XCB_NONE,
                                   XCB_RANDR_ROTATION_ROTATE_0,
@@ -429,24 +451,17 @@ apply_crtc_assignments (GfMonitorManager  *manager,
         }
     }
 
-  /* Disable CRTCs not mentioned in the list */
-  for (l = gf_gpu_get_crtcs (xrandr->gpu); l; l = l->next)
+  for (l = to_disable_crtcs; l; l = l->next)
     {
       GfCrtc *crtc = l->data;
 
-      if (crtc->is_dirty)
-        {
-          crtc->is_dirty = FALSE;
-          continue;
-        }
-
-      if (!crtc->config)
+      if (!gf_crtc_get_config (crtc))
         continue;
 
       xrandr_set_crtc_config (xrandr,
                               crtc,
                               save_timestamp,
-                              (xcb_randr_crtc_t) crtc->crtc_id,
+                              (xcb_randr_crtc_t) gf_crtc_get_id (crtc),
                               XCB_CURRENT_TIME,
                               0, 0, XCB_NONE,
                               XCB_RANDR_ROTATION_ROTATE_0,
@@ -469,59 +484,73 @@ apply_crtc_assignments (GfMonitorManager  *manager,
 
   for (i = 0; i < n_crtcs; i++)
     {
-      GfCrtcInfo *crtc_info = crtcs[i];
-      GfCrtc *crtc = crtc_info->crtc;
+      GfCrtcAssignment *crtc_assignment = crtcs[i];
+      GfCrtc *crtc = crtc_assignment->crtc;
 
-      if (crtc_info->mode != NULL)
+      if (crtc_assignment->mode != NULL)
         {
-          GfCrtcMode *mode;
+          GfCrtcMode *crtc_mode;
           xcb_randr_output_t *output_ids;
           guint j, n_output_ids;
           xcb_randr_rotation_t rotation;
+          xcb_randr_mode_t mode;
 
-          mode = crtc_info->mode;
+          crtc_mode = crtc_assignment->mode;
 
-          n_output_ids = crtc_info->outputs->len;
+          n_output_ids = crtc_assignment->outputs->len;
           output_ids = g_new0 (xcb_randr_output_t, n_output_ids);
 
           for (j = 0; j < n_output_ids; j++)
             {
               GfOutput *output;
+              GfOutputAssignment *output_assignment;
 
-              output = ((GfOutput**) crtc_info->outputs->pdata)[j];
+              output = ((GfOutput**) crtc_assignment->outputs->pdata)[j];
 
-              output->is_dirty = TRUE;
-              gf_output_assign_crtc (output, crtc);
+              to_configure_outputs = g_list_remove (to_configure_outputs, output);
 
-              output_ids[j] = output->winsys_id;
+              output_assignment = gf_find_output_assignment (outputs, n_outputs, output);
+              gf_output_assign_crtc (output, crtc, output_assignment);
+
+              output_ids[j] = gf_output_get_id (output);
             }
 
-          rotation = gf_monitor_transform_to_xrandr (crtc_info->transform);
+          rotation = gf_monitor_transform_to_xrandr (crtc_assignment->transform);
+          mode = gf_crtc_mode_get_id (crtc_mode);
+
           if (!xrandr_set_crtc_config (xrandr,
                                        crtc,
                                        save_timestamp,
-                                       (xcb_randr_crtc_t) crtc->crtc_id,
+                                       (xcb_randr_crtc_t) gf_crtc_get_id (crtc),
                                        XCB_CURRENT_TIME,
-                                       crtc_info->layout.x,
-                                       crtc_info->layout.y,
-                                       (xcb_randr_mode_t) mode->mode_id,
+                                       crtc_assignment->layout.x,
+                                       crtc_assignment->layout.y,
+                                       mode,
                                        rotation,
                                        output_ids, n_output_ids))
             {
+              const GfCrtcModeInfo *crtc_mode_info;
+
+              crtc_mode_info = gf_crtc_mode_get_info (crtc_mode);
+
               g_warning ("Configuring CRTC %d with mode %d (%d x %d @ %f) at position %d, %d and transform %u failed\n",
-                         (guint) (crtc->crtc_id), (guint) (mode->mode_id),
-                         mode->width, mode->height, (gdouble) mode->refresh_rate,
-                         crtc_info->layout.x, crtc_info->layout.y,
-                         crtc_info->transform);
+                         (unsigned) gf_crtc_get_id (crtc),
+                         (unsigned) mode,
+                         crtc_mode_info->width,
+                         crtc_mode_info->height,
+                         (double) crtc_mode_info->refresh_rate,
+                         crtc_assignment->layout.x,
+                         crtc_assignment->layout.y,
+                         crtc_assignment->transform);
 
               g_free (output_ids);
               continue;
             }
 
           gf_crtc_set_config (crtc,
-                              &crtc_info->layout,
-                              mode,
-                              crtc_info->transform);
+                              &crtc_assignment->layout,
+                              crtc_mode,
+                              crtc_assignment->transform);
 
           g_free (output_ids);
         }
@@ -529,33 +558,24 @@ apply_crtc_assignments (GfMonitorManager  *manager,
 
   for (i = 0; i < n_outputs; i++)
     {
-      GfOutputInfo *output_info = outputs[i];
-      GfOutput *output = output_info->output;
+      GfOutputAssignment *output_assignment = outputs[i];
+      GfOutput *output = output_assignment->output;
 
-      output->is_primary = output_info->is_primary;
-      output->is_presentation = output_info->is_presentation;
-      output->is_underscanning = output_info->is_underscanning;
-
-      gf_output_xrandr_apply_mode (output);
+      gf_output_xrandr_apply_mode (GF_OUTPUT_XRANDR (output));
     }
 
-  /* Disable outputs not mentioned in the list */
-  for (l = gf_gpu_get_outputs (xrandr->gpu); l; l = l->next)
+  for (l = to_configure_outputs; l; l = l->next)
     {
       GfOutput *output = l->data;
 
-      if (output->is_dirty)
-        {
-          output->is_dirty = FALSE;
-          continue;
-        }
-
       gf_output_unassign_crtc (output);
-      output->is_primary = FALSE;
     }
 
   XUngrabServer (xrandr->xdisplay);
   XFlush (xrandr->xdisplay);
+
+  g_clear_pointer (&to_configure_outputs, g_list_free);
+  g_clear_pointer (&to_disable_crtcs, g_list_free);
 }
 
 static GQuark
@@ -665,9 +685,6 @@ gf_monitor_manager_xrandr_constructed (GObject *object)
   xrandr->xdisplay = gf_backend_x11_get_xdisplay (GF_BACKEND_X11 (backend));
   xrandr->xroot = DefaultRootWindow (xrandr->xdisplay);
 
-  xrandr->gpu = GF_GPU (gf_gpu_xrandr_new (xrandr));
-  gf_monitor_manager_add_gpu (GF_MONITOR_MANAGER (xrandr), xrandr->gpu);
-
   if (XRRQueryExtension (xrandr->xdisplay, &rr_event_base, &rr_error_base))
     {
       gint major_version;
@@ -718,7 +735,6 @@ gf_monitor_manager_xrandr_finalize (GObject *object)
 
   xrandr = GF_MONITOR_MANAGER_XRANDR (object);
 
-  g_clear_object (&xrandr->gpu);
   g_clear_pointer (&xrandr->supported_scales, g_free);
 
   G_OBJECT_CLASS (gf_monitor_manager_xrandr_parent_class)->finalize (object);
@@ -728,7 +744,7 @@ static GBytes *
 gf_monitor_manager_xrandr_read_edid (GfMonitorManager *manager,
                                      GfOutput         *output)
 {
-  return gf_output_xrandr_read_edid (output);
+  return gf_output_xrandr_read_edid (GF_OUTPUT_XRANDR (output));
 }
 
 static void
@@ -806,8 +822,8 @@ gf_monitor_manager_xrandr_apply_monitors_config (GfMonitorManager        *manage
                                                  GfMonitorsConfigMethod   method,
                                                  GError                 **error)
 {
-  GPtrArray *crtc_infos;
-  GPtrArray *output_infos;
+  GPtrArray *crtc_assignments;
+  GPtrArray *output_assignments;
 
   if (!config)
     {
@@ -816,7 +832,7 @@ gf_monitor_manager_xrandr_apply_monitors_config (GfMonitorManager        *manage
     }
 
   if (!gf_monitor_config_manager_assign (manager, config,
-                                         &crtc_infos, &output_infos,
+                                         &crtc_assignments, &output_assignments,
                                          error))
     return FALSE;
 
@@ -831,17 +847,17 @@ gf_monitor_manager_xrandr_apply_monitors_config (GfMonitorManager        *manage
        * just update the logical state.
        */
       if (is_assignments_changed (manager,
-                                  (GfCrtcInfo **) crtc_infos->pdata,
-                                  crtc_infos->len,
-                                  (GfOutputInfo **) output_infos->pdata,
-                                  output_infos->len))
+                                  (GfCrtcAssignment **) crtc_assignments->pdata,
+                                  crtc_assignments->len,
+                                  (GfOutputAssignment **) output_assignments->pdata,
+                                  output_assignments->len))
         {
           apply_crtc_assignments (manager,
                                   TRUE,
-                                  (GfCrtcInfo **) crtc_infos->pdata,
-                                  crtc_infos->len,
-                                  (GfOutputInfo **) output_infos->pdata,
-                                  output_infos->len);
+                                  (GfCrtcAssignment **) crtc_assignments->pdata,
+                                  crtc_assignments->len,
+                                  (GfOutputAssignment **) output_assignments->pdata,
+                                  output_assignments->len);
         }
       else
         {
@@ -849,8 +865,8 @@ gf_monitor_manager_xrandr_apply_monitors_config (GfMonitorManager        *manage
         }
     }
 
-  g_ptr_array_free (crtc_infos, TRUE);
-  g_ptr_array_free (output_infos, TRUE);
+  g_ptr_array_free (crtc_assignments, TRUE);
+  g_ptr_array_free (output_assignments, TRUE);
 
   return TRUE;
 }
@@ -896,7 +912,7 @@ gf_monitor_manager_xrandr_change_backlight (GfMonitorManager *manager,
                                             GfOutput         *output,
                                             gint              value)
 {
-  gf_output_xrandr_change_backlight (output, value);
+  gf_output_xrandr_change_backlight (GF_OUTPUT_XRANDR (output), value);
 }
 
 static void
@@ -911,7 +927,7 @@ gf_monitor_manager_xrandr_get_crtc_gamma (GfMonitorManager  *manager,
   XRRCrtcGamma *gamma;
 
   xrandr = GF_MONITOR_MANAGER_XRANDR (manager);
-  gamma = XRRGetCrtcGamma (xrandr->xdisplay, (XID) crtc->crtc_id);
+  gamma = XRRGetCrtcGamma (xrandr->xdisplay, (XID) gf_crtc_get_id (crtc));
 
   *size = gamma->size;
   *red = g_memdup (gamma->red, sizeof (gushort) * gamma->size);
@@ -939,7 +955,7 @@ gf_monitor_manager_xrandr_set_crtc_gamma (GfMonitorManager *manager,
   memcpy (gamma->green, green, sizeof (gushort) * size);
   memcpy (gamma->blue, blue, sizeof (gushort) * size);
 
-  XRRSetCrtcGamma (xrandr->xdisplay, (XID) crtc->crtc_id, gamma);
+  XRRSetCrtcGamma (xrandr->xdisplay, (XID) gf_crtc_get_id (crtc), gamma);
   XRRFreeGamma (gamma);
 }
 
@@ -992,7 +1008,7 @@ gf_monitor_manager_xrandr_tiled_monitor_added (GfMonitorManager *manager,
     {
       GfOutput *output = l->data;
 
-      monitor_info->outputs[i] = output->winsys_id;
+      monitor_info->outputs[i] = gf_output_get_id (output);
     }
 
   XRRSetMonitor (xrandr->xdisplay, xrandr->xroot, monitor_info);
@@ -1026,7 +1042,7 @@ gf_monitor_manager_xrandr_is_transform_handled (GfMonitorManager   *manager,
                                                 GfCrtc             *crtc,
                                                 GfMonitorTransform  transform)
 {
-  g_warn_if_fail ((crtc->all_transforms & transform) == transform);
+  g_warn_if_fail ((gf_crtc_get_all_transforms (crtc) & transform) == transform);
 
   return TRUE;
 }
@@ -1060,14 +1076,7 @@ gf_monitor_manager_xrandr_calculate_supported_scales (GfMonitorManager
 static GfMonitorManagerCapability
 gf_monitor_manager_xrandr_get_capabilities (GfMonitorManager *manager)
 {
-  GfMonitorManagerCapability capabilities;
-
-  capabilities = GF_MONITOR_MANAGER_CAPABILITY_NONE;
-
-  capabilities |= GF_MONITOR_MANAGER_CAPABILITY_MIRRORING;
-  capabilities |= GF_MONITOR_MANAGER_CAPABILITY_GLOBAL_SCALE_REQUIRED;
-
-  return capabilities;
+  return GF_MONITOR_MANAGER_CAPABILITY_GLOBAL_SCALE_REQUIRED;
 }
 
 static gboolean
@@ -1076,10 +1085,12 @@ gf_monitor_manager_xrandr_get_max_screen_size (GfMonitorManager *manager,
                                                gint             *max_height)
 {
   GfMonitorManagerXrandr *xrandr;
+  GfGpu *gpu;
 
   xrandr = GF_MONITOR_MANAGER_XRANDR (manager);
+  gpu = get_gpu (xrandr);
 
-  gf_gpu_xrandr_get_max_screen_size (GF_GPU_XRANDR (xrandr->gpu),
+  gf_gpu_xrandr_get_max_screen_size (GF_GPU_XRANDR (gpu),
                                      max_width, max_height);
 
   return TRUE;
@@ -1144,6 +1155,7 @@ gf_monitor_manager_xrandr_handle_xevent (GfMonitorManagerXrandr *xrandr,
                                          XEvent                 *event)
 {
   GfMonitorManager *manager;
+  GfGpu *gpu;
   GfGpuXrandr *gpu_xrandr;
   XRRScreenResources *resources;
 
@@ -1155,7 +1167,8 @@ gf_monitor_manager_xrandr_handle_xevent (GfMonitorManagerXrandr *xrandr,
   XRRUpdateConfiguration (event);
   gf_monitor_manager_read_current_state (manager);
 
-  gpu_xrandr = GF_GPU_XRANDR (xrandr->gpu);
+  gpu = get_gpu (xrandr);
+  gpu_xrandr = GF_GPU_XRANDR (gpu);
   resources = gf_gpu_xrandr_get_resources (gpu_xrandr);
 
   if (!resources)
