@@ -36,27 +36,33 @@ generate_modes (GfMonitorNormal *normal)
 {
   GfMonitor *monitor;
   GfOutput *output;
+  const GfOutputInfo *output_info;
+  GfCrtcMode *preferred_mode;
   GfCrtcModeFlag preferred_mode_flags;
   guint i;
 
   monitor = GF_MONITOR (normal);
   output = gf_monitor_get_main_output (monitor);
-  preferred_mode_flags = output->preferred_mode->flags;
+  output_info = gf_output_get_info (output);
+  preferred_mode = output_info->preferred_mode;
+  preferred_mode_flags = gf_crtc_mode_get_info (preferred_mode)->flags;
 
-  for (i = 0; i < output->n_modes; i++)
+  for (i = 0; i < output_info->n_modes; i++)
     {
       GfCrtcMode *crtc_mode;
+      const GfCrtcModeInfo *crtc_mode_info;
       GfMonitorMode *mode;
       gboolean replace;
       GfCrtc *crtc;
 
-      crtc_mode = output->modes[i];
+      crtc_mode = output_info->modes[i];
+      crtc_mode_info = gf_crtc_mode_get_info (crtc_mode);
 
       mode = g_new0 (GfMonitorMode, 1);
       mode->monitor = monitor;
       mode->spec = gf_monitor_create_spec (monitor,
-                                           crtc_mode->width,
-                                           crtc_mode->height,
+                                           crtc_mode_info->width,
+                                           crtc_mode_info->height,
                                            crtc_mode);
 
       mode->id = gf_monitor_mode_spec_generate_id (&mode->spec);
@@ -72,21 +78,28 @@ generate_modes (GfMonitorNormal *normal)
        * otherwise take the first one in the list. This guarantees that the
        * preferred mode is always added.
        */
-      replace = crtc_mode->flags == preferred_mode_flags;
+      replace = crtc_mode_info->flags == preferred_mode_flags;
 
       if (!gf_monitor_add_mode (monitor, mode, replace))
         {
-          g_assert (crtc_mode != output->preferred_mode);
+          g_assert (crtc_mode != output_info->preferred_mode);
           gf_monitor_mode_free (mode);
           continue;
         }
 
-      if (crtc_mode == output->preferred_mode)
+      if (crtc_mode == output_info->preferred_mode)
         gf_monitor_set_preferred_mode (monitor, mode);
 
       crtc = gf_output_get_assigned_crtc (output);
-      if (crtc && crtc->config && crtc_mode == crtc->config->mode)
-        gf_monitor_set_current_mode (monitor, mode);
+
+      if (crtc != NULL)
+        {
+          const GfCrtcConfig *crtc_config;
+
+          crtc_config = gf_crtc_get_config (crtc);
+          if (crtc_config && crtc_mode == crtc_config->mode)
+            gf_monitor_set_current_mode (monitor, mode);
+        }
     }
 }
 
@@ -106,13 +119,15 @@ gf_monitor_normal_derive_layout (GfMonitor   *monitor,
 {
   GfOutput *output;
   GfCrtc *crtc;
+  const GfCrtcConfig *crtc_config;
 
   output = gf_monitor_get_main_output (monitor);
   crtc = gf_output_get_assigned_crtc (output);
+  crtc_config = gf_crtc_get_config (crtc);
 
-  g_return_if_fail (crtc->config);
+  g_return_if_fail (crtc_config);
 
-  *layout = crtc->config->layout;
+  *layout = crtc_config->layout;
 }
 
 static void
@@ -132,14 +147,18 @@ gf_monitor_normal_get_suggested_position (GfMonitor *monitor,
                                           gint      *x,
                                           gint      *y)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
-  if (output->suggested_x < 0 && output->suggested_y < 0)
+  output_info = gf_monitor_get_main_output_info (monitor);
+
+  if (!output_info->hotplug_mode_update)
     return FALSE;
 
-  *x = output->suggested_x;
-  *y = output->suggested_y;
+  if (output_info->suggested_x < 0 && output_info->suggested_y < 0)
+    return FALSE;
+
+  *x = output_info->suggested_x;
+  *y = output_info->suggested_y;
 
   return TRUE;
 }
@@ -176,7 +195,7 @@ gf_monitor_normal_new (GfGpu    *gpu,
   monitor = GF_MONITOR (normal);
 
   gf_monitor_append_output (monitor, output);
-  gf_monitor_set_winsys_id (monitor, output->winsys_id);
+  gf_monitor_set_winsys_id (monitor, gf_output_get_id (output));
   gf_monitor_generate_spec (monitor);
   generate_modes (normal);
 
