@@ -22,9 +22,11 @@
 #ifndef GF_MONITOR_PRIVATE_H
 #define GF_MONITOR_PRIVATE_H
 
+#include "gf-crtc-mode-private.h"
 #include "gf-gpu-private.h"
 #include "gf-monitor-manager-enums-private.h"
 #include "gf-monitor-manager-types-private.h"
+#include "gf-output-info-private.h"
 #include "gf-rectangle.h"
 
 G_BEGIN_DECLS
@@ -113,6 +115,8 @@ GfMonitorModeSpec   gf_monitor_create_spec                (GfMonitor
                                                            int                         height,
                                                            GfCrtcMode                 *crtc_mode);
 
+const GfOutputInfo *gf_monitor_get_main_output_info       (GfMonitor                  *self);
+
 void                gf_monitor_generate_spec              (GfMonitor                  *monitor);
 
 gboolean            gf_monitor_add_mode                   (GfMonitor                  *monitor,
