@@ -208,7 +208,7 @@ is_current_mode_known (GfMonitor *monitor)
   output = gf_monitor_get_main_output (monitor);
   crtc = gf_output_get_assigned_crtc (output);
 
-  return gf_monitor_is_active (monitor) == (crtc && crtc->config);
+  return gf_monitor_is_active (monitor) == (crtc && gf_crtc_get_config (crtc));
 }
 
 static gboolean
@@ -511,11 +511,13 @@ void
 gf_monitor_make_display_name (GfMonitor *monitor)
 {
   GfMonitorPrivate *priv;
+  GfBackend *backend;
   GfMonitorManager *manager;
 
   priv = gf_monitor_get_instance_private (monitor);
 
-  manager = gf_gpu_get_monitor_manager (priv->gpu);
+  backend = gf_gpu_get_backend (priv->gpu);
+  manager = gf_backend_get_monitor_manager (backend);
 
   g_free (priv->display_name);
   priv->display_name = make_display_name (monitor, manager);
@@ -546,14 +548,16 @@ gf_monitor_is_mode_assigned (GfMonitor     *monitor,
       GfOutput *output;
       GfMonitorCrtcMode *monitor_crtc_mode;
       GfCrtc *crtc;
+      const GfCrtcConfig *crtc_config;
 
       output = l->data;
       monitor_crtc_mode = &mode->crtc_modes[i];
       crtc = gf_output_get_assigned_crtc (output);
+      crtc_config = crtc ? gf_crtc_get_config (crtc) : NULL;
 
       if (monitor_crtc_mode->crtc_mode &&
-          (!crtc || !crtc->config ||
-           crtc->config->mode != monitor_crtc_mode->crtc_mode))
+          (!crtc || !crtc_config ||
+           crtc_config->mode != monitor_crtc_mode->crtc_mode))
         return FALSE;
       else if (!monitor_crtc_mode->crtc_mode && crtc)
         return FALSE;
@@ -601,12 +605,14 @@ gf_monitor_create_spec (GfMonitor  *monitor,
                         int         height,
                         GfCrtcMode *crtc_mode)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
+  const GfCrtcModeInfo *crtc_mode_info;
   GfMonitorModeSpec spec;
 
-  output = gf_monitor_get_main_output (monitor);
+  output_info = gf_monitor_get_main_output_info (monitor);
+  crtc_mode_info = gf_crtc_mode_get_info (crtc_mode);
 
-  if (gf_monitor_transform_is_rotated (output->panel_orientation_transform))
+  if (gf_monitor_transform_is_rotated (output_info->panel_orientation_transform))
     {
       int temp;
 
@@ -617,28 +623,38 @@ gf_monitor_create_spec (GfMonitor  *monitor,
 
   spec.width = width;
   spec.height = height;
-  spec.refresh_rate = crtc_mode->refresh_rate;
-  spec.flags = crtc_mode->flags & HANDLED_CRTC_MODE_FLAGS;
+  spec.refresh_rate = crtc_mode_info->refresh_rate;
+  spec.flags = crtc_mode_info->flags & HANDLED_CRTC_MODE_FLAGS;
 
   return spec;
 }
 
+const GfOutputInfo *
+gf_monitor_get_main_output_info (GfMonitor *self)
+{
+  GfOutput *output;
+
+  output = gf_monitor_get_main_output (self);
+
+  return gf_output_get_info (output);
+}
+
 void
 gf_monitor_generate_spec (GfMonitor *monitor)
 {
   GfMonitorPrivate *priv;
-  GfOutput *output;
+  const GfOutputInfo *output_info;
   GfMonitorSpec *monitor_spec;
 
   priv = gf_monitor_get_instance_private (monitor);
-  output = gf_monitor_get_main_output (monitor);
+  output_info = gf_monitor_get_main_output_info (monitor);
 
   monitor_spec = g_new0 (GfMonitorSpec, 1);
 
-  monitor_spec->connector = g_strdup (output->name);
-  monitor_spec->vendor = g_strdup (output->vendor);
-  monitor_spec->product = g_strdup (output->product);
-  monitor_spec->serial = g_strdup (output->serial);
+  monitor_spec->connector = g_strdup (output_info->name);
+  monitor_spec->vendor = g_strdup (output_info->vendor);
+  monitor_spec->product = g_strdup (output_info->product);
+  monitor_spec->serial = g_strdup (output_info->serial);
 
   priv->spec = monitor_spec;
 }
@@ -722,17 +738,17 @@ gf_monitor_is_primary (GfMonitor *monitor)
 
   output = gf_monitor_get_main_output (monitor);
 
-  return output->is_primary;
+  return gf_output_is_primary (output);
 }
 
 gboolean
 gf_monitor_supports_underscanning (GfMonitor *monitor)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
+  output_info = gf_monitor_get_main_output_info (monitor);
 
-  return output->supports_underscanning;
+  return output_info->supports_underscanning;
 }
 
 gboolean
@@ -742,17 +758,17 @@ gf_monitor_is_underscanning (GfMonitor *monitor)
 
   output = gf_monitor_get_main_output (monitor);
 
-  return output->is_underscanning;
+  return gf_output_is_underscanning (output);
 }
 
 gboolean
 gf_monitor_is_laptop_panel (GfMonitor *monitor)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
+  output_info = gf_monitor_get_main_output_info (monitor);
 
-  switch (output->connector_type)
+  switch (output_info->connector_type)
     {
       case GF_CONNECTOR_TYPE_eDP:
       case GF_CONNECTOR_TYPE_LVDS:
@@ -826,57 +842,62 @@ gf_monitor_get_physical_dimensions (GfMonitor *monitor,
                                     gint      *width_mm,
                                     gint      *height_mm)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
+  output_info = gf_monitor_get_main_output_info (monitor);
 
-  *width_mm = output->width_mm;
-  *height_mm = output->height_mm;
+  *width_mm = output_info->width_mm;
+  *height_mm = output_info->height_mm;
 }
 
 const gchar *
 gf_monitor_get_connector (GfMonitor *monitor)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
-  return output->name;
+  output_info = gf_monitor_get_main_output_info (monitor);
+
+  return output_info->name;
 }
 
 const gchar *
 gf_monitor_get_vendor (GfMonitor *monitor)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
-  return output->vendor;
+  output_info = gf_monitor_get_main_output_info (monitor);
+
+  return output_info->vendor;
 }
 
 const gchar *
 gf_monitor_get_product (GfMonitor *monitor)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
-  return output->product;
+  output_info = gf_monitor_get_main_output_info (monitor);
+
+  return output_info->product;
 }
 
 const gchar *
 gf_monitor_get_serial (GfMonitor *monitor)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
-  return output->serial;
+  output_info = gf_monitor_get_main_output_info (monitor);
+
+  return output_info->serial;
 }
 
 GfConnectorType
 gf_monitor_get_connector_type (GfMonitor *monitor)
 {
-  GfOutput *output;
+  const GfOutputInfo *output_info;
 
-  output = gf_monitor_get_main_output (monitor);
-  return output->connector_type;
+  output_info = gf_monitor_get_main_output_info (monitor);
+
+  return output_info->connector_type;
 }
 
 GfMonitorTransform
@@ -1041,7 +1062,8 @@ gf_monitor_calculate_mode_scale (GfMonitor     *monitor,
   gint global_scaling_factor;
 
   priv = gf_monitor_get_instance_private (monitor);
-  monitor_manager = gf_gpu_get_monitor_manager (priv->gpu);
+  backend = gf_gpu_get_backend (priv->gpu);
+  monitor_manager = gf_backend_get_monitor_manager (backend);
   backend = gf_monitor_manager_get_backend (monitor_manager);
   settings = gf_backend_get_settings (backend);
 
