@@ -32,19 +32,33 @@
 #include <xcb/randr.h>
 
 #include "gf-crtc-private.h"
+#include "gf-monitor-manager-xrandr-private.h"
+
+struct _GfOutputXrandr
+{
+  GfOutput parent;
+};
+
+G_DEFINE_TYPE (GfOutputXrandr, gf_output_xrandr, GF_TYPE_OUTPUT)
+
+static Display *
+xdisplay_from_gpu (GfGpu *gpu)
+{
+  GfBackend *backend;
+  GfMonitorManager *monitor_manager;
+  GfMonitorManagerXrandr *monitor_manager_xrandr;
+
+  backend = gf_gpu_get_backend (gpu);
+  monitor_manager = gf_backend_get_monitor_manager (backend);
+  monitor_manager_xrandr = GF_MONITOR_MANAGER_XRANDR (monitor_manager);
+
+  return gf_monitor_manager_xrandr_get_xdisplay (monitor_manager_xrandr);
+}
 
 static Display *
 xdisplay_from_output (GfOutput *output)
 {
-  GfGpu *gpu;
-  GfMonitorManager *monitor_manager;
-  GfMonitorManagerXrandr *monitor_manager_xrandr;
-
-  gpu = gf_output_get_gpu (output);
-  monitor_manager = gf_gpu_get_monitor_manager (gpu);
-  monitor_manager_xrandr = GF_MONITOR_MANAGER_XRANDR (monitor_manager);
-
-  return gf_monitor_manager_xrandr_get_xdisplay (monitor_manager_xrandr);
+  return xdisplay_from_gpu (gf_output_get_gpu (output));
 }
 
 static void
@@ -60,7 +74,7 @@ output_set_presentation_xrandr (GfOutput *output,
   value= presentation;
 
   xcb_randr_change_output_property (XGetXCBConnection (xdisplay),
-                                    (XID) output->winsys_id,
+                                    (XID) gf_output_get_id (output),
                                     atom, XCB_ATOM_CARDINAL, 32,
                                     XCB_PROP_MODE_REPLACE,
                                     1, &value);
@@ -81,7 +95,7 @@ output_set_underscanning_xrandr (GfOutput *output,
   valueatom = XInternAtom (xdisplay, value, False);
 
   xcb_randr_change_output_property (XGetXCBConnection (xdisplay),
-                                    (XID) output->winsys_id,
+                                    (XID) gf_output_get_id (output),
                                     prop, XCB_ATOM_ATOM, 32,
                                     XCB_PROP_MODE_REPLACE,
                                     1, &valueatom);
@@ -93,26 +107,28 @@ output_set_underscanning_xrandr (GfOutput *output,
   if (underscanning)
     {
       GfCrtc *crtc;
-      GfCrtcConfig *crtc_config;
+      const GfCrtcConfig *crtc_config;
+      const GfCrtcModeInfo *crtc_mode_info;
       uint32_t border_value;
 
       crtc = gf_output_get_assigned_crtc (output);
-      crtc_config = crtc->config;
+      crtc_config = gf_crtc_get_config (crtc);
+      crtc_mode_info = gf_crtc_mode_get_info (crtc_config->mode);
 
       prop = XInternAtom (xdisplay, "underscan hborder", False);
-      border_value = crtc_config->mode->width * 0.05;
+      border_value = crtc_mode_info->width * 0.05;
 
       xcb_randr_change_output_property (XGetXCBConnection (xdisplay),
-                                        (XID) output->winsys_id,
+                                        (XID) gf_output_get_id (output),
                                         prop, XCB_ATOM_INTEGER, 32,
                                         XCB_PROP_MODE_REPLACE,
                                         1, &border_value);
 
       prop = XInternAtom (xdisplay, "underscan vborder", False);
-      border_value = crtc_config->mode->height * 0.05;
+      border_value = crtc_mode_info->height * 0.05;
 
       xcb_randr_change_output_property (XGetXCBConnection (xdisplay),
-                                        (XID) output->winsys_id,
+                                        (XID) gf_output_get_id (output),
                                         prop, XCB_ATOM_INTEGER, 32,
                                         XCB_PROP_MODE_REPLACE,
                                         1, &border_value);
@@ -154,20 +170,47 @@ get_edid_property (Display  *xdisplay,
   return result;
 }
 
+static GBytes *
+read_xrandr_edid (Display  *xdisplay,
+                  RROutput  output_id)
+{
+  Atom edid_atom;
+  guint8 *result;
+  gsize len;
+
+  edid_atom = XInternAtom (xdisplay, "EDID", FALSE);
+  result = get_edid_property (xdisplay, output_id, edid_atom, &len);
+
+  if (!result)
+    {
+      edid_atom = XInternAtom (xdisplay, "EDID_DATA", FALSE);
+      result = get_edid_property (xdisplay, output_id, edid_atom,  &len);
+    }
+
+  if (result)
+    {
+      if (len > 0 && len % 128 == 0)
+        return g_bytes_new_take (result, len);
+      else
+        g_free (result);
+    }
+
+  return NULL;
+}
+
 static gboolean
-output_get_property_exists (GfOutput    *output,
+output_get_property_exists (Display     *xdisplay,
+                            RROutput     output_id,
                             const gchar *propname)
 {
-  Display *xdisplay;
   gboolean exists;
   Atom atom, actual_type;
   gint actual_format;
   gulong nitems, bytes_after;
   guchar *buffer;
 
-  xdisplay = xdisplay_from_output (output);
   atom = XInternAtom (xdisplay, propname, False);
-  XRRGetOutputProperty (xdisplay, (XID) output->winsys_id, atom,
+  XRRGetOutputProperty (xdisplay, (XID) output_id, atom,
                         0, G_MAXLONG, False, False, AnyPropertyType,
                         &actual_type, &actual_format,
                         &nitems, &bytes_after, &buffer);
@@ -181,26 +224,26 @@ output_get_property_exists (GfOutput    *output,
 }
 
 static gboolean
-output_get_hotplug_mode_update (GfOutput *output)
+output_get_hotplug_mode_update (Display  *xdisplay,
+                                RROutput  output_id)
 {
-  return output_get_property_exists (output, "hotplug_mode_update");
+  return output_get_property_exists (xdisplay, output_id, "hotplug_mode_update");
 }
 
 static gboolean
-output_get_integer_property (GfOutput    *output,
+output_get_integer_property (Display     *xdisplay,
+                             RROutput     output_id,
                              const gchar *propname,
                              gint        *value)
 {
-  Display *xdisplay;
   gboolean exists;
   Atom atom, actual_type;
   gint actual_format;
   gulong nitems, bytes_after;
   guchar *buffer;
 
-  xdisplay = xdisplay_from_output (output);
   atom = XInternAtom (xdisplay, propname, False);
-  XRRGetOutputProperty (xdisplay, (XID) output->winsys_id, atom,
+  XRRGetOutputProperty (xdisplay, (XID) output_id, atom,
                         0, G_MAXLONG, False, False, XA_INTEGER,
                         &actual_type, &actual_format,
                         &nitems, &bytes_after, &buffer);
@@ -217,22 +260,24 @@ output_get_integer_property (GfOutput    *output,
 }
 
 static gint
-output_get_suggested_x (GfOutput *output)
+output_get_suggested_x (Display  *xdisplay,
+                        RROutput  output_id)
 {
   gint val;
 
-  if (output_get_integer_property (output, "suggested X", &val))
+  if (output_get_integer_property (xdisplay, output_id, "suggested X", &val))
     return val;
 
   return -1;
 }
 
 static gint
-output_get_suggested_y (GfOutput *output)
+output_get_suggested_y (Display  *xdisplay,
+                        RROutput  output_id)
 {
   gint val;
 
-  if (output_get_integer_property (output, "suggested Y", &val))
+  if (output_get_integer_property (xdisplay, output_id, "suggested Y", &val))
     return val;
 
   return -1;
@@ -275,18 +320,17 @@ connector_type_from_atom (Display *xdisplay,
 }
 
 static GfConnectorType
-output_get_connector_type_from_prop (GfOutput *output)
+output_get_connector_type_from_prop (Display  *xdisplay,
+                                     RROutput  output_id)
 {
-  Display *xdisplay;
   Atom atom, actual_type, connector_type_atom;
   gint actual_format;
   gulong nitems, bytes_after;
   guchar *buffer;
   GfConnectorType ret;
 
-  xdisplay = xdisplay_from_output (output);
   atom = XInternAtom (xdisplay, "ConnectorType", False);
-  XRRGetOutputProperty (xdisplay, (XID) output->winsys_id, atom,
+  XRRGetOutputProperty (xdisplay, (XID) output_id, atom,
                         0, G_MAXLONG, False, False, XA_ATOM,
                         &actual_type, &actual_format,
                         &nitems, &bytes_after, &buffer);
@@ -307,11 +351,11 @@ output_get_connector_type_from_prop (GfOutput *output)
 }
 
 static GfConnectorType
-output_get_connector_type_from_name (GfOutput *output)
+output_info_get_connector_type_from_name (const GfOutputInfo *output_info)
 {
   const gchar *name;
 
-  name = output->name;
+  name = output_info->name;
 
   /* drmmode_display.c, which was copy/pasted across all the FOSS
    * xf86-video-* drivers, seems to name its outputs based on the
@@ -357,7 +401,9 @@ output_get_connector_type_from_name (GfOutput *output)
 }
 
 static GfConnectorType
-output_get_connector_type (GfOutput *output)
+output_info_get_connector_type (GfOutputInfo *output_info,
+                                Display      *xdisplay,
+                                RROutput      output_id)
 {
   GfConnectorType ret;
 
@@ -368,12 +414,12 @@ output_get_connector_type (GfOutput *output)
    * Try poking it first, without any expectations that it will work.
    * If it's not there, we thankfully have other bonghits to try next.
    */
-  ret = output_get_connector_type_from_prop (output);
+  ret = output_get_connector_type_from_prop (xdisplay, output_id);
   if (ret != GF_CONNECTOR_TYPE_Unknown)
     return ret;
 
   /* Fall back to heuristics based on the output name. */
-  ret = output_get_connector_type_from_name (output);
+  ret = output_info_get_connector_type_from_name (output_info);
   if (ret != GF_CONNECTOR_TYPE_Unknown)
     return ret;
 
@@ -381,9 +427,9 @@ output_get_connector_type (GfOutput *output)
 }
 
 static GfMonitorTransform
-output_get_panel_orientation_transform (GfOutput *output)
+output_get_panel_orientation_transform (Display  *xdisplay,
+                                        RROutput  output_id)
 {
-  Display *xdisplay;
   unsigned long nitems;
   unsigned long bytes_after;
   Atom atom;
@@ -393,12 +439,11 @@ output_get_panel_orientation_transform (GfOutput *output)
   char *str;
   GfMonitorTransform transform;
 
-  xdisplay = xdisplay_from_output (output);
   buffer = NULL;
   str = NULL;
 
   atom = XInternAtom (xdisplay, "panel orientation", False);
-  XRRGetOutputProperty (xdisplay, (XID) output->winsys_id, atom,
+  XRRGetOutputProperty (xdisplay, (XID) output_id, atom,
                         0, G_MAXLONG, False, False, XA_ATOM,
                         &actual_type, &actual_format,
                         &nitems, &bytes_after, &buffer);
@@ -427,28 +472,18 @@ output_get_panel_orientation_transform (GfOutput *output)
 }
 
 static void
-output_get_tile_info (GfOutput *output)
+output_info_init_tile_info (GfOutputInfo *output_info,
+                            Display      *xdisplay,
+                            RROutput      output_id)
 {
-  GfGpu *gpu;
-  GfMonitorManager *monitor_manager;
-  GfMonitorManagerXrandr *monitor_manager_xrandr;
-  Display *xdisplay;
   Atom tile_atom;
   guchar *prop;
   gulong nitems, bytes_after;
   gint actual_format;
   Atom actual_type;
 
-  gpu = gf_output_get_gpu (output);
-  monitor_manager = gf_gpu_get_monitor_manager (gpu);
-  monitor_manager_xrandr = GF_MONITOR_MANAGER_XRANDR (monitor_manager);
-
-  if (!gf_monitor_manager_xrandr_has_randr15 (monitor_manager_xrandr))
-    return;
-
-  xdisplay = gf_monitor_manager_xrandr_get_xdisplay (monitor_manager_xrandr);
   tile_atom = XInternAtom (xdisplay, "TILE", FALSE);
-  XRRGetOutputProperty (xdisplay, output->winsys_id,
+  XRRGetOutputProperty (xdisplay, (XID) output_id,
                         tile_atom, 0, 100, False,
                         False, AnyPropertyType,
                         &actual_type, &actual_format,
@@ -457,14 +492,15 @@ output_get_tile_info (GfOutput *output)
   if (actual_type == XA_INTEGER && actual_format == 32 && nitems == 8)
     {
       glong *values = (glong *) prop;
-      output->tile_info.group_id = values[0];
-      output->tile_info.flags = values[1];
-      output->tile_info.max_h_tiles = values[2];
-      output->tile_info.max_v_tiles = values[3];
-      output->tile_info.loc_h_tile = values[4];
-      output->tile_info.loc_v_tile = values[5];
-      output->tile_info.tile_w = values[6];
-      output->tile_info.tile_h = values[7];
+
+      output_info->tile_info.group_id = values[0];
+      output_info->tile_info.flags = values[1];
+      output_info->tile_info.max_h_tiles = values[2];
+      output_info->tile_info.max_v_tiles = values[3];
+      output_info->tile_info.loc_h_tile = values[4];
+      output_info->tile_info.loc_v_tile = values[5];
+      output_info->tile_info.tile_w = values[6];
+      output_info->tile_info.tile_h = values[7];
     }
 
   if (prop)
@@ -472,16 +508,14 @@ output_get_tile_info (GfOutput *output)
 }
 
 static void
-output_get_modes (GfOutput      *output,
-                  XRROutputInfo *xrandr_output)
+output_info_init_modes (GfOutputInfo  *output_info,
+                        GfGpu         *gpu,
+                        XRROutputInfo *xrandr_output)
 {
-  GfGpu *gpu;
   guint j;
   guint n_actual_modes;
 
-  gpu = gf_output_get_gpu (output);
-
-  output->modes = g_new0 (GfCrtcMode *, xrandr_output->nmode);
+  output_info->modes = g_new0 (GfCrtcMode *, xrandr_output->nmode);
 
   n_actual_modes = 0;
   for (j = 0; j < (guint) xrandr_output->nmode; j++)
@@ -492,32 +526,30 @@ output_get_modes (GfOutput      *output,
         {
           GfCrtcMode *mode = l->data;
 
-          if (xrandr_output->modes[j] == (XID) mode->mode_id)
+          if (xrandr_output->modes[j] == (XID) gf_crtc_mode_get_id (mode))
             {
-              output->modes[n_actual_modes] = mode;
+              output_info->modes[n_actual_modes] = mode;
               n_actual_modes += 1;
               break;
             }
         }
     }
 
-  output->n_modes = n_actual_modes;
+  output_info->n_modes = n_actual_modes;
   if (n_actual_modes > 0)
-    output->preferred_mode = output->modes[0];
+    output_info->preferred_mode = output_info->modes[0];
 }
 
 static void
-output_get_crtcs (GfOutput      *output,
-                  XRROutputInfo *xrandr_output)
+output_info_init_crtcs (GfOutputInfo  *output_info,
+                        GfGpu         *gpu,
+                        XRROutputInfo *xrandr_output)
 {
-  GfGpu *gpu;
   guint j;
   guint n_actual_crtcs;
   GList *l;
 
-  gpu = gf_output_get_gpu (output);
-
-  output->possible_crtcs = g_new0 (GfCrtc *, xrandr_output->ncrtc);
+  output_info->possible_crtcs = g_new0 (GfCrtc *, xrandr_output->ncrtc);
 
   n_actual_crtcs = 0;
   for (j = 0; j < (guint) xrandr_output->ncrtc; j++)
@@ -526,27 +558,32 @@ output_get_crtcs (GfOutput      *output,
         {
           GfCrtc *crtc = l->data;
 
-          if ((XID) crtc->crtc_id == xrandr_output->crtcs[j])
+          if ((XID) gf_crtc_get_id (crtc) == xrandr_output->crtcs[j])
             {
-              output->possible_crtcs[n_actual_crtcs] = crtc;
+              output_info->possible_crtcs[n_actual_crtcs] = crtc;
               n_actual_crtcs += 1;
               break;
             }
         }
     }
-  output->n_possible_crtcs = n_actual_crtcs;
+  output_info->n_possible_crtcs = n_actual_crtcs;
+}
+
+static GfCrtc *
+find_assigned_crtc (GfGpu         *gpu,
+                    XRROutputInfo *xrandr_output)
+{
+  GList *l;
 
-  gf_output_unassign_crtc (output);
   for (l = gf_gpu_get_crtcs (gpu); l; l = l->next)
     {
       GfCrtc *crtc = l->data;
 
-      if ((XID) crtc->crtc_id == xrandr_output->crtc)
-        {
-          gf_output_assign_crtc (output, crtc);
-          break;
-        }
+      if ((XID) gf_crtc_get_id (crtc) == xrandr_output->crtc)
+        return crtc;
     }
+
+  return NULL;
 }
 
 static gboolean
@@ -562,7 +599,7 @@ output_get_boolean_property (GfOutput    *output,
 
   xdisplay = xdisplay_from_output (output);
   atom = XInternAtom (xdisplay, propname, False);
-  XRRGetOutputProperty (xdisplay, (XID) output->winsys_id, atom,
+  XRRGetOutputProperty (xdisplay, (XID) gf_output_get_id (output), atom,
                         0, G_MAXLONG, False, False, XA_CARDINAL,
                         &actual_type, &actual_format,
                         &nitems, &bytes_after, &buffer);
@@ -600,7 +637,7 @@ output_get_underscanning_xrandr (GfOutput *output)
 
   xdisplay = xdisplay_from_output (output);
   atom = XInternAtom (xdisplay, "underscan", False);
-  XRRGetOutputProperty (xdisplay, (XID) output->winsys_id, atom,
+  XRRGetOutputProperty (xdisplay, (XID) gf_output_get_id (output), atom,
                         0, G_MAXLONG, False, False, XA_ATOM,
                         &actual_type, &actual_format,
                         &nitems, &bytes_after, &buffer);
@@ -623,9 +660,9 @@ output_get_underscanning_xrandr (GfOutput *output)
 }
 
 static gboolean
-output_get_supports_underscanning_xrandr (GfOutput *output)
+output_get_supports_underscanning_xrandr (Display  *xdisplay,
+                                          RROutput  output_id)
 {
-  Display *xdisplay;
   Atom atom, actual_type;
   gint actual_format, i;
   gulong nitems, bytes_after;
@@ -634,9 +671,8 @@ output_get_supports_underscanning_xrandr (GfOutput *output)
   Atom *values;
   gboolean supports_underscanning = FALSE;
 
-  xdisplay = xdisplay_from_output (output);
   atom = XInternAtom (xdisplay, "underscan", False);
-  XRRGetOutputProperty (xdisplay, (XID) output->winsys_id, atom,
+  XRRGetOutputProperty (xdisplay, (XID) output_id, atom,
                         0, G_MAXLONG, False, False, XA_ATOM,
                         &actual_type, &actual_format,
                         &nitems, &bytes_after, &buffer);
@@ -649,9 +685,7 @@ output_get_supports_underscanning_xrandr (GfOutput *output)
       return FALSE;
     }
 
-  property_info = XRRQueryOutputProperty (xdisplay,
-                                          (XID) output->winsys_id,
-                                          atom);
+  property_info = XRRQueryOutputProperty (xdisplay, (XID) output_id, atom);
   values = (Atom *) property_info->values;
 
   for (i = 0; i < property_info->num_values; i++)
@@ -675,8 +709,12 @@ static int
 normalize_backlight (GfOutput *output,
                      gint      hw_value)
 {
-  return round ((gdouble) (hw_value - output->backlight_min) /
-                (output->backlight_max - output->backlight_min) * 100.0);
+  const GfOutputInfo *output_info;
+
+  output_info = gf_output_get_info (output);
+
+  return round ((double) (hw_value - output_info->backlight_min) /
+                (output_info->backlight_max - output_info->backlight_min) * 100.0);
 }
 
 static gint
@@ -691,7 +729,7 @@ output_get_backlight_xrandr (GfOutput *output)
 
   xdisplay = xdisplay_from_output (output);
   atom = XInternAtom (xdisplay, "Backlight", False);
-  XRRGetOutputProperty (xdisplay, (XID) output->winsys_id, atom,
+  XRRGetOutputProperty (xdisplay, (XID) gf_output_get_id (output), atom,
                         0, G_MAXLONG, False, False, XA_INTEGER,
                         &actual_type, &actual_format,
                         &nitems, &bytes_after, &buffer);
@@ -714,21 +752,21 @@ output_get_backlight_xrandr (GfOutput *output)
 }
 
 static void
-output_get_backlight_limits_xrandr (GfOutput *output)
+output_info_init_backlight_limits_xrandr (GfOutputInfo       *output_info,
+                                          Display            *xdisplay,
+                                          xcb_randr_output_t  output_id)
 {
-  Display *xdisplay;
   Atom atom;
   xcb_connection_t *xcb_conn;
   xcb_randr_query_output_property_cookie_t cookie;
   xcb_randr_query_output_property_reply_t *reply;
   int32_t *values;
 
-  xdisplay = xdisplay_from_output (output);
   atom = XInternAtom (xdisplay, "Backlight", False);
 
   xcb_conn = XGetXCBConnection (xdisplay);
   cookie = xcb_randr_query_output_property (xcb_conn,
-                                            (xcb_randr_output_t) output->winsys_id,
+                                            output_id,
                                             (xcb_atom_t) atom);
 
   reply = xcb_randr_query_output_property_reply (xcb_conn, cookie, NULL);
@@ -739,62 +777,89 @@ output_get_backlight_limits_xrandr (GfOutput *output)
 
   if (!reply->range || reply->length != 2)
     {
-      g_warning ("backlight %s was not range\n", output->name);
+      g_warning ("backlight %s was not range\n", output_info->name);
       g_free (reply);
       return;
     }
 
   values = xcb_randr_query_output_property_valid_values (reply);
 
-  output->backlight_min = values[0];
-  output->backlight_max = values[1];
+  output_info->backlight_min = values[0];
+  output_info->backlight_max = values[1];
 
   g_free (reply);
 }
 
-GfOutput *
-gf_create_xrandr_output (GfGpuXrandr   *gpu_xrandr,
-                         XRROutputInfo *xrandr_output,
-                         RROutput       output_id,
-                         RROutput       primary_output)
+static void
+gf_output_xrandr_class_init (GfOutputXrandrClass *self_class)
 {
+}
+
+static void
+gf_output_xrandr_init (GfOutputXrandr *self)
+{
+}
+
+GfOutputXrandr *
+gf_output_xrandr_new (GfGpuXrandr   *gpu_xrandr,
+                      XRROutputInfo *xrandr_output,
+                      RROutput       output_id,
+                      RROutput       primary_output)
+{
+  GfGpu *gpu;
+  GfBackend *backend;
+  GfMonitorManager *monitor_manager;
+  GfMonitorManagerXrandr *monitor_manager_xrandr;
+  Display *xdisplay;
+  GfOutputInfo *output_info;
   GfOutput *output;
   GBytes *edid;
+  GfCrtc *assigned_crtc;
   unsigned int i;
 
-  output = g_object_new (GF_TYPE_OUTPUT, NULL);
-  output->gpu = GF_GPU (gpu_xrandr);
+  gpu = GF_GPU (gpu_xrandr);
+  backend = gf_gpu_get_backend (gpu);
+  monitor_manager = gf_backend_get_monitor_manager (backend);
+  monitor_manager_xrandr = GF_MONITOR_MANAGER_XRANDR (monitor_manager);
+  xdisplay = gf_monitor_manager_xrandr_get_xdisplay (monitor_manager_xrandr);
 
-  output->winsys_id = output_id;
-  output->name = g_strdup (xrandr_output->name);
+  output_info = gf_output_info_new ();
 
-  edid = gf_output_xrandr_read_edid (output);
-  gf_output_parse_edid (output, edid);
+  output_info->name = g_strdup (xrandr_output->name);
+
+  edid = read_xrandr_edid (xdisplay, output_id);
+  gf_output_info_parse_edid (output_info, edid);
   g_bytes_unref (edid);
 
-  output->hotplug_mode_update = output_get_hotplug_mode_update (output);
-  output->suggested_x = output_get_suggested_x (output);
-  output->suggested_y = output_get_suggested_y (output);
-  output->connector_type = output_get_connector_type (output);
-  output->panel_orientation_transform = output_get_panel_orientation_transform (output);
+  output_info->hotplug_mode_update = output_get_hotplug_mode_update (xdisplay,
+                                                                     output_id);
+  output_info->suggested_x = output_get_suggested_x (xdisplay, output_id);
+  output_info->suggested_y = output_get_suggested_y (xdisplay, output_id);
+  output_info->connector_type = output_info_get_connector_type (output_info,
+                                                                xdisplay,
+                                                                output_id);
+  output_info->panel_orientation_transform = output_get_panel_orientation_transform (xdisplay,
+                                                                                     output_id);
 
-  if (gf_monitor_transform_is_rotated (output->panel_orientation_transform))
+  if (gf_monitor_transform_is_rotated (output_info->panel_orientation_transform))
     {
-      output->width_mm = xrandr_output->mm_height;
-      output->height_mm = xrandr_output->mm_width;
+      output_info->width_mm = xrandr_output->mm_height;
+      output_info->height_mm = xrandr_output->mm_width;
     }
   else
     {
-      output->width_mm = xrandr_output->mm_width;
-      output->height_mm = xrandr_output->mm_height;
+      output_info->width_mm = xrandr_output->mm_width;
+      output_info->height_mm = xrandr_output->mm_height;
     }
 
-  output_get_tile_info (output);
-  output_get_modes (output, xrandr_output);
-  output_get_crtcs (output, xrandr_output);
+  if (!gf_monitor_manager_xrandr_has_randr15 (monitor_manager_xrandr))
+    output_info_init_tile_info (output_info, xdisplay, output_id);
 
-  output->n_possible_clones = xrandr_output->nclone;
-  output->possible_clones = g_new0 (GfOutput *, output->n_possible_clones);
+  output_info_init_modes (output_info, gpu, xrandr_output);
+  output_info_init_crtcs (output_info, gpu, xrandr_output);
+
+  output_info->n_possible_clones = xrandr_output->nclone;
+  output_info->possible_clones = g_new0 (GfOutput *, output_info->n_possible_clones);
 
   /* We can build the list of clones now, because we don't have
    * the list of outputs yet, so temporarily set the pointers to
@@ -802,96 +867,113 @@ gf_create_xrandr_output (GfGpuXrandr   *gpu_xrandr,
    */
   for (i = 0; i < (unsigned int) xrandr_output->nclone; i++)
     {
-      output->possible_clones[i] = GINT_TO_POINTER (xrandr_output->clones[i]);
+      output_info->possible_clones[i] = GINT_TO_POINTER (xrandr_output->clones[i]);
     }
 
-  output->is_primary = ((XID) output->winsys_id == primary_output);
-  output->is_presentation = output_get_presentation_xrandr (output);
-  output->is_underscanning = output_get_underscanning_xrandr (output);
-  output->supports_underscanning = output_get_supports_underscanning_xrandr (output);
+  output_info->supports_underscanning = output_get_supports_underscanning_xrandr (xdisplay,
+                                                                                  output_id);
+  output_info_init_backlight_limits_xrandr (output_info, xdisplay, output_id);
 
-  output_get_backlight_limits_xrandr (output);
+  output = g_object_new (GF_TYPE_OUTPUT_XRANDR,
+                         "id", output_id,
+                         "gpu", gpu_xrandr,
+                         "info", output_info,
+                         NULL);
 
-  if (!(output->backlight_min == 0 && output->backlight_max == 0))
-    output->backlight = output_get_backlight_xrandr (output);
-  else
-    output->backlight = -1;
-
-  if (output->n_modes == 0 || output->n_possible_crtcs == 0)
+  assigned_crtc = find_assigned_crtc (gpu, xrandr_output);
+  if (assigned_crtc)
     {
+      GfOutputAssignment output_assignment;
+
+      output_assignment = (GfOutputAssignment) {
+        .is_primary = (XID) gf_output_get_id (output) == primary_output,
+        .is_presentation = output_get_presentation_xrandr (output),
+        .is_underscanning = output_get_underscanning_xrandr (output),
+      };
+
+      gf_output_assign_crtc (output, assigned_crtc, &output_assignment);
+    }
+  else
+    {
+      gf_output_unassign_crtc (output);
+    }
+
+  if (!(output_info->backlight_min == 0 && output_info->backlight_max == 0))
+    gf_output_set_backlight (output, output_get_backlight_xrandr (output));
+
+  if (output_info->n_modes == 0 || output_info->n_possible_crtcs == 0)
+    {
+      gf_output_info_unref (output_info);
       g_object_unref (output);
       return NULL;
     }
 
-  return output;
+  gf_output_info_unref (output_info);
+
+  return GF_OUTPUT_XRANDR (output);
 }
 
 GBytes *
-gf_output_xrandr_read_edid (GfOutput *output)
+gf_output_xrandr_read_edid (GfOutputXrandr *self)
 {
+  GfOutput *output;
   Display *xdisplay;
-  Atom edid_atom;
-  guint8 *result;
-  gsize len;
+  RROutput output_id;
 
+  output = GF_OUTPUT (self);
   xdisplay = xdisplay_from_output (output);
-  edid_atom = XInternAtom (xdisplay, "EDID", FALSE);
-  result = get_edid_property (xdisplay, output->winsys_id, edid_atom, &len);
+  output_id = (RROutput) gf_output_get_id (output);
 
-  if (!result)
-    {
-      edid_atom = XInternAtom (xdisplay, "EDID_DATA", FALSE);
-      result = get_edid_property (xdisplay, output->winsys_id, edid_atom, &len);
-    }
-
-  if (result)
-    {
-      if (len > 0 && len % 128 == 0)
-        return g_bytes_new_take (result, len);
-      else
-        g_free (result);
-    }
-
-  return NULL;
+  return read_xrandr_edid (xdisplay, output_id);
 }
 
 void
-gf_output_xrandr_apply_mode (GfOutput *output)
+gf_output_xrandr_apply_mode (GfOutputXrandr *self)
 {
+  GfOutput *output;
   Display *xdisplay;
 
+  output = GF_OUTPUT (self);
   xdisplay = xdisplay_from_output (output);
 
-  if (output->is_primary)
+  if (gf_output_is_primary (output))
     {
       XRRSetOutputPrimary (xdisplay, DefaultRootWindow (xdisplay),
-                           (XID) output->winsys_id);
+                           (XID) gf_output_get_id (output));
     }
 
-  output_set_presentation_xrandr (output, output->is_presentation);
+  output_set_presentation_xrandr (output, gf_output_is_presentation (output));
 
-  if (output->supports_underscanning)
-    output_set_underscanning_xrandr (output, output->is_underscanning);
+  if (gf_output_get_info (output)->supports_underscanning)
+    {
+      output_set_underscanning_xrandr (output,
+                                       gf_output_is_underscanning (output));
+    }
 }
 
 void
-gf_output_xrandr_change_backlight (GfOutput *output,
-                                   int       value)
+gf_output_xrandr_change_backlight (GfOutputXrandr *self,
+                                   int             value)
 {
+  GfOutput *output;
+  const GfOutputInfo *output_info;
   Display *xdisplay;
   gint hw_value;
   Atom atom;
 
+  output = GF_OUTPUT (self);
+
+  output_info = gf_output_get_info (output);
   xdisplay = xdisplay_from_output (output);
-  hw_value = round ((gdouble) value / 100.0 * output->backlight_max + output->backlight_min);
+  hw_value = round ((double) value / 100.0 * output_info->backlight_max + output_info->backlight_min);
   atom = XInternAtom (xdisplay, "Backlight", False);
 
   xcb_randr_change_output_property (XGetXCBConnection (xdisplay),
-                                    (XID)output->winsys_id,
+                                    (XID) gf_output_get_id (output),
                                     atom, XCB_ATOM_INTEGER, 32,
                                     XCB_PROP_MODE_REPLACE,
                                     1, &hw_value);
 
   /* We're not selecting for property notifies, so update the value immediately */
-  output->backlight = normalize_backlight (output, hw_value);
+  gf_output_set_backlight (output, normalize_backlight (output, hw_value));
 }
